@@ -48,6 +48,7 @@ typedef struct CLEANUP_STATE {
     VSTRING *attr_buf;			/* storage for named attribute */
     VSTRING *temp1;			/* scratch buffer, local use only */
     VSTRING *temp2;			/* scratch buffer, local use only */
+    VSTRING *temp3;			/* scratch buffer, local use only */
     VSTRING *stripped_buf;		/* character stripped input */
     VSTREAM *src;			/* current input stream */
     VSTREAM *dst;			/* current output stream */
@@ -128,9 +129,9 @@ typedef struct CLEANUP_STATE {
     struct CLEANUP_REGION *curr_body_region;
 
     /*
-     * Internationalization.
+     * Internationalization, RequireTLS, etc.
      */
-    int     smtputf8;			/* what support is desired */
+    int     sendopts;			/* what support is desired */
 } CLEANUP_STATE;
 
  /*
@@ -368,4 +369,7 @@ extern int cleanup_hfrom_format;
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
