@@ -6,7 +6,8 @@
 /* SYNOPSIS
 /*	#include "cleanup.h"
 /*
-/*	CLEANUP_STATE *cleanup_open()
+/*	CLEANUP_STATE *cleanup_open(src)
+/*	VSTREAM	*src;
 /*
 /*	void	cleanup_control(state, flags)
 /*	CLEANUP_STATE *state;
@@ -46,6 +47,10 @@
 /*	Enable header/body filtering. This should be enabled only with mail
 /*	that enters Postfix, not with locally forwarded mail or with bounce
 /*	messages.
+/* .IP CLEANUP_FLAG_MILTER
+/*	Enable Milter applications. This should be enabled only with mail
+/*	that enters Postfix, not with locally forwarded mail or with bounce
+/*	messages.
 /* .IP CLEANUP_FLAG_MAP_OK
 /*	Enable canonical and virtual mapping, and address masquerading.
 /* .PP
@@ -70,8 +75,14 @@
 /*	to translate the result into human-readable text.
 /*
 /*	cleanup_free() destroys its argument.
+/* .IP CLEANUP_FLAG_SMTPUTF8
+/*	Request SMTPUTF8 support when delivering mail.
+/* .IP CLEANUP_FLAG_AUTOUTF8
+/*	Autodetection: request SMTPUTF8 support if the message
+/*	contains an UTF8 message header, sender, or recipient.
 /* DIAGNOSTICS
-/*	Problems and transactions are logged to \fBsyslogd\fR(8).
+/*	Problems and transactions are logged to \fBsyslogd\fR(8)
+/*	or \fBpostlogd\fR(8).
 /* SEE ALSO
 /*	cleanup(8) cleanup service description.
 /*	cleanup_init(8) cleanup callable interface, initialization
@@ -84,6 +95,14 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -106,6 +125,12 @@
 #include <mail_params.h>
 #include <mail_stream.h>
 #include <mail_flow.h>
+#include <rec_type.h>
+#include <smtputf8.h>
+
+/* Milter library. */
+
+#include <milter.h>
 
 /* Application-specific. */
 
@@ -113,21 +138,21 @@
 
 /* cleanup_open - open queue file and initialize */
 
-CLEANUP_STATE *cleanup_open(void)
+CLEANUP_STATE *cleanup_open(VSTREAM *src)
 {
     CLEANUP_STATE *state;
-    static char *log_queues[] = {
+    static const char *log_queues[] = {
 	MAIL_QUEUE_DEFER,
 	MAIL_QUEUE_BOUNCE,
 	MAIL_QUEUE_TRACE,
 	0,
     };
-    char  **cpp;
+    const char **cpp;
 
     /*
      * Initialize private state.
      */
-    state = cleanup_state_alloc();
+    state = cleanup_state_alloc(src);
 
     /*
      * Open the queue file. Save the queue file name in a global variable, so
@@ -183,6 +208,9 @@ void    cleanup_control(CLEANUP_STATE *state, int flags)
     } else {
 	state->err_mask = ~0;
     }
+    if (state->flags & CLEANUP_FLAG_SMTPUTF8)
+	state->sendopts |= SMTPUTF8_FLAG_REQUESTED;
+    /* TODO: REQUIRETLS */
 }
 
 /* cleanup_flush - finish queue file */
@@ -210,13 +238,35 @@ int     cleanup_flush(CLEANUP_STATE *state)
     if (state->flags & CLEANUP_FLAG_DISCARD)
 	state->errs = 0;
 
+    /*
+     * Apply external mail filter.
+     * 
+     * XXX Include test for a built-in action to tempfail this message.
+     */
+    if (CLEANUP_MILTER_OK(state)) {
+	if (state->milters)
+	    cleanup_milter_inspect(state, state->milters);
+	else if (cleanup_milters) {
+	    cleanup_milter_emul_data(state, cleanup_milters);
+	    if (CLEANUP_MILTER_OK(state))
+		cleanup_milter_inspect(state, cleanup_milters);
+	}
+    }
+
+    /*
+     * Update the preliminary message size and count fields with the actual
+     * values.
+     */
+    if (CLEANUP_OUT_OK(state))
+	cleanup_final(state);
+
     /*
      * If there was an error that requires us to generate a bounce message
      * (mail submitted with the Postfix sendmail command, mail forwarded by
      * the local(8) delivery agent, or mail re-queued with "postsuper -r"),
      * send a bounce notification, reset the error flags in case of success,
-     * and request deletion of the the incoming queue file and of the
-     * optional DSN SUCCESS records from virtual alias expansion.
+     * and request deletion of the incoming queue file and of the optional
+     * DSN SUCCESS records from virtual alias expansion.
      * 
      * XXX It would make no sense to knowingly report success after we already
      * have bounced all recipients, especially because the information in the
@@ -247,6 +297,8 @@ int     cleanup_flush(CLEANUP_STATE *state)
      * (or else the queue manager would grab it too early) and updating our
      * own idea of the queue file name for error recovery and for error
      * reporting purposes.
+     * 
+     * XXX Include test for a built-in action to tempfail this message.
      */
     if (state->errs == 0 && (state->flags & CLEANUP_FLAG_DISCARD) == 0) {
 	if ((state->flags & CLEANUP_FLAG_HOLD) != 0
@@ -262,13 +314,13 @@ int     cleanup_flush(CLEANUP_STATE *state)
 	    state->queue_name = mystrdup(MAIL_QUEUE_HOLD);
 #endif
 	    mail_stream_ctl(state->handle,
-			    MAIL_STREAM_CTL_QUEUE, state->queue_name,
-			    MAIL_STREAM_CTL_CLASS, 0,
-			    MAIL_STREAM_CTL_SERVICE, 0,
+			    CA_MAIL_STREAM_CTL_QUEUE(state->queue_name),
+			    CA_MAIL_STREAM_CTL_CLASS((char *) 0),
+			    CA_MAIL_STREAM_CTL_SERVICE((char *) 0),
 #ifdef DELAY_ACTION
-			    MAIL_STREAM_CTL_DELAY, state->defer_delay,
+			    CA_MAIL_STREAM_CTL_DELAY(state->defer_delay),
 #endif
-			    MAIL_STREAM_CTL_END);
+			    CA_MAIL_STREAM_CTL_END);
 	    junk = cleanup_path;
 	    cleanup_path = mystrdup(VSTREAM_PATH(state->handle->stream));
 	    myfree(junk);
@@ -277,12 +329,20 @@ int     cleanup_flush(CLEANUP_STATE *state)
 	     * XXX: When delivering to a non-incoming queue, do not consume
 	     * in_flow tokens. Unfortunately we can't move the code that
 	     * consumes tokens until after the mail is received, because that
-	     * would increase the risk of duplicate deliveries.
+	     * would increase the risk of duplicate deliveries (RFC 1047).
 	     */
 	    (void) mail_flow_put(1);
 	}
 	state->errs = mail_stream_finish(state->handle, (VSTRING *) 0);
     } else {
+
+	/*
+	 * XXX: When discarding mail, should we consume in_flow tokens? See
+	 * also the comments above for mail that is placed on hold.
+	 */
+#if 0
+	(void) mail_flow_put(1);
+#endif
 	mail_stream_cleanup(state->handle);
     }
     state->handle = 0;
@@ -298,6 +358,8 @@ int     cleanup_flush(CLEANUP_STATE *state)
 	    (void) REMOVE(vstring_str(cleanup_trace_path));
 	if (REMOVE(cleanup_path))
 	    msg_warn("remove %s: %m", cleanup_path);
+	msg_info("%s: removed (%s)", state->queue_id, state->errs ?
+		 "canceled" : "discarded");
     }
 
     /*
@@ -328,5 +390,12 @@ int     cleanup_flush(CLEANUP_STATE *state)
 
 void    cleanup_free(CLEANUP_STATE *state)
 {
+
+    /*
+     * Emulate disconnect event. CLEANUP_FLAG_MILTER may be turned off after
+     * we have started.
+     */
+    if (cleanup_milters != 0 && state->milters == 0)
+	milter_disc_event(cleanup_milters);
     cleanup_state_free(state);
 }
