@@ -650,8 +650,8 @@ int     smtp_helo(SMTP_STATE *state)
      * non-SMTPUTF8 server? That could make life easier for mailing lists.
      */
 #define DELIVERY_REQUIRES_SMTPUTF8 \
-	((request->smtputf8 & SMTPUTF8_FLAG_REQUESTED) \
-	&& (request->smtputf8 & ~SMTPUTF8_FLAG_REQUESTED))
+	((request->sendopts & SMTPUTF8_FLAG_REQUESTED) \
+	&& (request->sendopts & SMTPUTF8_FLAG_DERIVED))
 
     /*
      * Require that the server supports SMTPUTF8 when delivery requires
@@ -1171,7 +1171,7 @@ static int smtp_start_tls(SMTP_STATE *state)
 #ifdef USE_TLSRPT
 
 	    /*
-	     * Don't create a TLSRPT failure report here, if the TLS engine
+	     * Don't create a TLSRPT 'failure' event here, if the TLS engine
 	     * already reported a more specific reason.
 	     */
 	    if (state->tlsrpt && session->tls_context->rpt_reported == 0) {
@@ -1194,15 +1194,14 @@ static int smtp_start_tls(SMTP_STATE *state)
 	}
 
     /*
-     * Create a TLSRPT success report only if the TLS engine has not reported
-     * a failure (For example, the TLS handshake may be successful, but the
-     * security level was downgraded from opportunistic or half "dane" to
-     * "encrypt"), and only if the TLS session and SMTP connection are new.
+     * Create a TLSRPT 'success' event only if the TLS engine has not created
+     * TLSRPT event. For example, The TLS engine will create a TLSRPT
+     * 'failure' event when the TLS handshake was be successful, but the
+     * security level was downgraded from opportunistic "dane" to
+     * unauthenticated "encrypt".
      */
 #ifdef USE_TLSRPT
-    if (state->tlsrpt && session->tls_context->rpt_reported == 0
-	&& session->tls_context->session_reused == 0	/* New TLS */
-	&& session->reuse_count == 0)		/* New SMTP */
+    if (state->tlsrpt && session->tls_context->rpt_reported == 0)
 	(void) trw_report_success(state->tlsrpt);
 #endif
 
@@ -1777,7 +1776,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	     * the SMTPUTF8 promise that was made to the sender.
 	     */
 	    if ((session->features & SMTP_FEATURE_SMTPUTF8) != 0
-		&& (request->smtputf8 & SMTPUTF8_FLAG_REQUESTED) != 0)
+		&& (request->sendopts & SMTPUTF8_FLAG_REQUESTED) != 0)
 		vstring_strcat(next_command, " SMTPUTF8");
 
 	    /*
@@ -1842,7 +1841,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 		    quote_822_local(session->scratch, rcpt->orig_addr);
 		    vstring_sprintf(session->scratch2, "%s;%s",
 		    /* Fix 20140707: sender must request SMTPUTF8. */
-				    (request->smtputf8 != 0
+				    ((request->sendopts & SMTPUTF8_FLAG_ALL)
 			      && !allascii(vstring_str(session->scratch))) ?
 				    "utf-8" : "rfc822",
 				    vstring_str(session->scratch));
